@@ -1,5 +1,5 @@
 /* Connect and Disconnect commands
-   Copyright (C) 2016-2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2016-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -319,12 +319,6 @@ get_server_v2(PIRC_SERVER ptr, const size_t size, const char *hdr)
 		fflush(stdout);
 		free(msg);
 
-/*
- * sscanf() is safe in this context
- */
-#if WIN32
-#pragma warning(disable: 4996)
-#endif
 		if (fgets(ans, sizeof ans, stdin) == NULL) {
 			err_sys("%s: fgets", __func__);
 		} else if (strchr(ans, '\n') == NULL) {
@@ -334,18 +328,12 @@ get_server_v2(PIRC_SERVER ptr, const size_t size, const char *hdr)
 
 			while (c = getchar(), c != '\n' && c != EOF)
 				/* discard */;
-		} else if (sscanf(ans, "%d", &srvno) != 1 || srvno < 0 ||
+		} else if (xsscanf(ans, "%d", &srvno) != 1 || srvno < 0 ||
 		    srvno > i) {
 			;
 		} else {
 			break;
 		}
-/*
- * Reset warning behavior to its default value
- */
-#if WIN32
-#pragma warning(default: 4996)
-#endif
 	} /* while */
 
 	UNUSED_PARAM(size);
@@ -437,19 +425,6 @@ assign_nickname(char **cp)
 	return false;
 }
 
-static void
-add_cmd(PTEXTBUF matches, const char *str)
-{
-	if (textBuf_size(matches) != 0) {
-		if ((errno = textBuf_ins_next(matches, textBuf_tail(matches),
-		    str, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	} else {
-		if ((errno = textBuf_ins_next(matches, NULL, str, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	}
-}
-
 PTEXTBUF
 get_list_of_matching_connect_cmds(const char *search_var)
 {
@@ -460,7 +435,7 @@ get_list_of_matching_connect_cmds(const char *search_var)
 		const char *cmd = connect_cmds[i];
 
 		if (!strncmp(search_var, cmd, varlen))
-			add_cmd(matches, cmd);
+			textBuf_emplace_back(__func__, matches, cmd, 0);
 	}
 	if (textBuf_size(matches) == 0) {
 		textBuf_destroy(matches);
