@@ -1,5 +1,5 @@
 /* commands/dcc.cpp
-   Copyright (C) 2024, 2025 Markus Uhlin. All rights reserved.
+   Copyright (C) 2024-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -50,6 +50,7 @@ namespace fs = std::filesystem;
 #if WIN32
 #include <io.h>
 #endif
+#include <limits.h>
 #include <stdexcept>
 #if UNIX
 #include <unistd.h>
@@ -204,7 +205,7 @@ read_and_write(SOCKET sock, SSL *ssl, FILE *fp, intmax_t &bytes_rem)
 		if (!isValid(ssl))
 			throw std::runtime_error("ssl object invalid");
 		if ((ret = SSL_read(ssl, addrof(buf[0]), (bytes_rem < bufsize ?
-		    bytes_rem : bufsize))) > 0) {
+		    static_cast<int>(bytes_rem) : bufsize))) > 0) {
 			if (!isValid(fp) ||
 			    fwrite(addrof(buf[0]), 1, ret, fp) !=
 			    static_cast<size_t>(ret))
@@ -1007,7 +1008,8 @@ list_get(void)
 		printtext(&ctx, "%sHas completed%s: %s (%.2f%%)",
 		    COLOR2, TXT_NORMAL,
 		    (x.has_completed() ? _("Yes") : _("No")),
-		    percentage(x.filesize - x.bytes_rem, x.filesize));
+		    percentage(TO_DBL(x.filesize - x.bytes_rem),
+		    TO_DBL(x.filesize)));
 
 		get_time(str1, x.start);
 		get_time(str2, x.stop);
@@ -1049,7 +1051,8 @@ list_send(void)
 		printtext(&ctx, "%sHas completed%s: %s (%.2f%%)",
 		    COLOR2, TXT_NORMAL,
 		    (x.has_completed() ? _("Yes") : _("No")),
-		    percentage(filesize - x.bytes_rem, filesize));
+		    percentage(TO_DBL(filesize - x.bytes_rem),
+		    TO_DBL(filesize)));
 
 		objnum++;
 	}
@@ -1279,19 +1282,6 @@ list_dir(const char *dir)
 #endif
 }
 
-static void
-add_cmd(PTEXTBUF matches, const char *cmd)
-{
-	if (textBuf_size(matches) == 0) {
-		if ((errno = textBuf_ins_next(matches, nullptr, cmd, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	} else {
-		if ((errno = textBuf_ins_next(matches, textBuf_tail(matches),
-		    cmd, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	}
-}
-
 PTEXTBUF
 get_list_of_matching_dcc_cmds(const char *search_var)
 {
@@ -1302,7 +1292,7 @@ get_list_of_matching_dcc_cmds(const char *search_var)
 		const char *cmd = dcc_cmds[i];
 
 		if (!strncmp(search_var, cmd, varlen))
-			add_cmd(matches, cmd);
+			textBuf_emplace_back(__func__, matches, cmd, 0);
 	}
 
 	if (textBuf_size(matches) == 0) {
@@ -1516,6 +1506,19 @@ dcc_deinit(void)
 	(void) napms(100);
 }
 
+int
+imax_to_int(const char *fn, const intmax_t i)
+{
+	if (i > INT_MAX) {
+		char buf[200] = { '\0' };
+
+		(void)snprintf(buf, sizeof buf, "%s: integer maximum exceeded",
+		    fn);
+		throw std::overflow_error(&buf[0]);
+	}
+	return static_cast<int>(i);
+}
+
 void
 dcc::add_file(const char *nick, const char *user, const char *host,
     const char *data)
@@ -1567,31 +1570,17 @@ dcc::add_file(const char *nick, const char *user, const char *host,
 		uint16_t	port = 0;
 		intmax_t	filesize = 0;
 
-/*
- * sscanf() is safe in this context
- */
-#if WIN32
-#pragma warning(disable: 4996)
-#endif
-
 		if (!(get_db.size() < GET_DB_MAX))
 			throw std::runtime_error("database full");
-		else if (sscanf(token[0], "%" SCNu32, &addr) != 1)
+		else if (xsscanf(token[0], "%" SCNu32, &addr) != 1)
 			throw std::runtime_error("error getting the address");
-		else if (sscanf(token[1], "%" SCNu16, &port) != 1)
+		else if (xsscanf(token[1], "%" SCNu16, &port) != 1)
 			throw std::runtime_error("error getting the port");
-		else if (sscanf(token[2], "%" SCNdMAX, &filesize) != 1)
+		else if (xsscanf(token[2], "%" SCNdMAX, &filesize) != 1)
 			throw std::runtime_error("error getting the file size");
 		else if (filesize > DCC_FILE_MAX_SIZE)
 			throw std::runtime_error("too large file");
 
-/*
- * Reset warning behavior to its default value
- */
-#if WIN32
-#pragma warning(default: 4996)
-#endif
-
 		nick_lc = strToLower(sw_strdup(nick));
 		dup_check_get(nick_lc, token[3]);
 
@@ -1846,7 +1835,7 @@ send_doit(SSL *ssl, dcc_send *send_obj)
 		static const int	bufsize = static_cast<int>(sizeof buf);
 
 		bytes = ((send_obj->bytes_rem < bufsize)
-			 ? send_obj->bytes_rem
+			 ? static_cast<int>(send_obj->bytes_rem)
 			 : bufsize);
 		if (!isValid(send_obj->fileptr))
 			break;
