@@ -1,5 +1,5 @@
 /* The FTP command
-   Copyright (C) 2024, 2025 Markus Uhlin. All rights reserved.
+   Copyright (C) 2024-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -282,12 +282,23 @@ handle_length_four(char (&numstr)[5], CSTRING token, const size_t len,
 	memcpy(numstr, token, len);
 	numstr[len] = '\0';
 
+	errno = 0;
+
 	if (!has_three_digits(numstr)) {
 		err_log(0, "%s: expected digits", __func__);
-	} else if (sscanf(numstr, "%d%c", &num, &ch) != 2 ||
-		   ch != '-') {
+	}
+#ifdef HAVE_BCI
+	else if (sscanf_s(numstr, "%d%c", &num, &ch, (UINT)sizeof(char)) != 2 ||
+		 ch != '-') {
+		err_log(errno, "%s: sscanf_s() error", __func__);
+	}
+#else
+	else if (sscanf(numstr, "%d%c", &num, &ch) != 2 ||
+		 ch != '-') {
 		err_log(0, "%s: sscanf() error", __func__);
-	} else {
+	}
+#endif
+	else {
 		FTP_REPLY rep(num, token + len);
 
 #if defined(__cplusplus) && __cplusplus >= 201103L
@@ -309,7 +320,7 @@ handle_length_three(char (&numstr)[5], CSTRING token, const size_t len,
 
 	if (!has_three_digits(numstr)) {
 		err_log(0, "%s: expected digits", __func__);
-	} else if (sscanf(numstr, "%d", &num) != 1) {
+	} else if (xsscanf(numstr, "%d", &num) != 1) {
 		err_log(0, "%s: sscanf() error", __func__);
 	} else {
 		FTP_REPLY rep(num, token + len);
@@ -456,7 +467,7 @@ ftp_data_conn::ftp_data_conn(CSTRING text)
 	static CSTRING format = " Entering Passive Mode "
 	    "(%hhu,%hhu,%hhu,%hhu,%hhu,%hhu).";
 
-	if (sscanf(text, format,
+	if (xsscanf(text, format,
 	    &this->h[0], &this->h[1], &this->h[2], &this->h[3],
 	    &this->p[0], &this->p[1]) != 6)
 		throw std::runtime_error("too few items assigned");
@@ -551,7 +562,7 @@ get_bytes(const std::string &str)
 
 	if ((bytes_str = strtok_r(nullptr, sep, &tokstate)) == nullptr ||
 	    !is_numeric(bytes_str) ||
-	    sscanf(bytes_str, "%jd", &ftp::data_conn->filesz) != 1) {
+	    xsscanf(bytes_str, "%jd", &ftp::data_conn->filesz) != 1) {
 		printtext_print("warn", "failed to get the file size");
 		ftp::data_conn->filesz = -1;
 	}
@@ -650,7 +661,8 @@ ftp_data_conn::get_file(void)
 			} else
 				total += bytes_received;
 			if (this->filesz != -1) {
-				print_complete(this->path, total, this->filesz,
+				print_complete(this->path,
+				    TO_DBL(total), TO_DBL(this->filesz),
 				    printed);
 			}
 		} else if (bytes_received == 0) {
@@ -805,7 +817,8 @@ ftp_data_conn::send_file(void)
 			bytes_rem -= bytes_sent;
 			total += bytes_sent;
 
-			print_complete(this->path, total, this->filesz,
+			print_complete(this->path,
+			    TO_DBL(total), TO_DBL(this->filesz),
 			    printed);
 		}
 	}
@@ -1182,19 +1195,6 @@ ftp_deinit(void)
 	ftp::data_conn = nullptr;
 }
 
-static void
-add_cmd(PTEXTBUF matches, CSTRING str)
-{
-	if (textBuf_size(matches) != 0) {
-		if ((errno = textBuf_ins_next(matches, textBuf_tail(matches),
-		    str, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	} else {
-		if ((errno = textBuf_ins_next(matches, nullptr, str, -1)) != 0)
-			err_sys("%s: textBuf_ins_next", __func__);
-	}
-}
-
 PTEXTBUF
 get_list_of_matching_ftp_cmds(CSTRING search_var)
 {
@@ -1205,7 +1205,7 @@ get_list_of_matching_ftp_cmds(CSTRING search_var)
 		CSTRING cmd = ftp_cmds[i];
 
 		if (!strncmp(search_var, cmd, varlen))
-			add_cmd(matches, cmd);
+			textBuf_emplace_back(__func__, matches, cmd, 0);
 	}
 
 	if (textBuf_size(matches) == 0) {
