@@ -21,7 +21,7 @@ __SWIRC_BEGIN_DECLS
 STRING		crypt_decrypt_str(CSTRING, cryptstr_const_t, const bool);
 STRING		crypt_encrypt_str(cryptstr_const_t, cryptstr_const_t,
 		    const bool);
-void		crypt_freezero(void *, size_t);
+void		crypt_freezero(void *, const size_t);
 int		crypt_get_base64_decode_length(CSTRING);
 uint32_t	crypt_get_base64_encode_length(const uint32_t);
 int		crypt_get_key_and_iv(cryptstr_const_t password, PCRYPT_CTX);
