@@ -430,6 +430,8 @@ event_names_htbl_lookup(CSTRING nick, CSTRING channel)
 int
 event_names_htbl_insert(CSTRING nick, CSTRING channel)
 {
+	int ret;
+
 	if (nick == nullptr || strings_match(nick, ""))
 		return ERR;
 
@@ -438,7 +440,11 @@ event_names_htbl_insert(CSTRING nick, CSTRING channel)
 	ctx.channel	= const_cast<STRING>(channel);
 	ctx.nick	= const_cast<STRING>(nick);
 
-	if (hInstall(&ctx) == ERR)
+	mutex_lock(&g_win_htbl_mtx);
+	ret = hInstall(&ctx);
+	mutex_unlock(&g_win_htbl_mtx);
+
+	if (ret == ERR)
 		return ERR;
 	if (nicklist_update(window_by_label(channel)) != 0)
 		debug("event_names_htbl_insert: nicklist_update: error");
@@ -459,7 +465,9 @@ event_names_htbl_remove(CSTRING nick, CSTRING channel)
 	    names != nullptr;
 	    names = names->next) {
 		if (strings_match_ignore_case(nick, names->nick)) {
+			mutex_lock(&g_win_htbl_mtx);
 			hUndef(window, names);
+			mutex_unlock(&g_win_htbl_mtx);
 
 			if (nicklist_update(window) != 0) {
 				debug("event_names_htbl_remove: "
@@ -629,7 +637,11 @@ event_names(struct irc_message_compo *compo)
 			struct hInstall_context ctx(channel, nick, (ret > 0
 			    ? privs : ""));
 
-			if (hInstall(&ctx) != OK)
+			mutex_lock(&g_win_htbl_mtx);
+			const int iret = hInstall(&ctx);
+			mutex_unlock(&g_win_htbl_mtx);
+
+			if (iret != OK)
 				continue;
 		}
 	} catch (const std::runtime_error &e) {
@@ -655,6 +667,9 @@ event_names(struct irc_message_compo *compo)
 	free(names_copy);
 }
 
+/*
+ * XXX: Do NOT lock/unlock 'g_win_htbl_mtx' here...
+ */
 void
 event_names_htbl_remove_all(PIRC_WINDOW window)
 {
