@@ -10,5 +10,7 @@
 #endif
 
 #define NO_SIDE_EFFECT
+#define FMT_STRFTIME(arg_no)
 #define PRINTFLIKE(arg_no)
+#define SCANFLIKE(arg_no)
 #define SW_INLINE inline
