@@ -88,7 +88,7 @@
 /* Things with external linkage
    ============================ */
 
-chararray_t	g_swircVersion	= "v3.5.5";
+chararray_t	g_swircVersion	= "v3.5.6";
 chararray_t	g_swircYear	= "2012-2025";
 chararray_t	g_swircAuthor	= "Markus Uhlin";
 chararray_t	g_swircWebAddr	= "https://www.nifty-networks.net/swirc/";
@@ -157,6 +157,7 @@ static stringarray_t OptionDesc = {
   N_("    -X                   Disable all IRCv3 extensions\n"),
   N_("    -c <server[:port]>   Connect to IRC server\n"),
   N_("    -d                   Debug logging\n"),
+  N_("    -h <hostname>        Sets the bind host\n"),
   N_("    -i                   Turn on Internet Citizen's Band mode\n"),
   N_("    -j <join>            A comma-separated list of channels to join\n"),
   N_("    -n <nickname>        Online nickname\n"),
@@ -342,6 +343,8 @@ case_hostname()
 
 	if (been_case)
 		DUP_OPTION_ERR('h');
+	else if (!is_valid_hostname(g_option_arg))
+		err_quit("invalid hostname: %s", g_option_arg);
 
 	g_cmdline_opts->hostname = sw_strdup(g_option_arg);
 	g_bind_hostname = been_case = true;
@@ -659,7 +662,7 @@ main(int argc, char *argv[])
 {
 	char *cp;
 
-#if __OpenBSD__
+#if defined(OpenBSD) && OpenBSD <= 202504
 	extern char *malloc_options;
 
 	malloc_options = const_cast<char *>("S");
