@@ -1,5 +1,5 @@
 /* Create the home directory and read its configuration files
-   Copyright (C) 2012-2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2012-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -104,7 +104,15 @@ init_globals(const char *hp)
 	g_home_dir = strdup_printf("%s/.swirc", hp);
 	g_tmp_dir = strdup_printf("%s/.swirc/tmp", hp);
 	g_log_dir = strdup_printf("%s/.swirc/log", hp);
+#if __OpenBSD__
+	/*
+	 * OpenBSD uses unveil()
+	 */
 	g_dcc_download_dir = strdup_printf("%s/.swirc/downloads", hp);
+#else
+	if ((g_dcc_download_dir = path_to_downloads()) == NULL)
+		g_dcc_download_dir = strdup_printf("%s/.swirc/downloads", hp);
+#endif
 	g_dcc_upload_dir = strdup_printf("%s/.swirc/uploads", hp);
 	g_config_file = strdup_printf("%s/.swirc/swirc%s", hp,
 	    g_config_filesuffix);
@@ -396,10 +404,17 @@ nestHome_deinit(void)
 char *
 path_to_downloads(void)
 {
+	char buf[PATH_MAX] = { '\0' };
+
 #if defined(UNIX)
-	return (NULL);
+	if (sw_strcpy(buf, path_to_home(), ARRAY_SIZE(buf)) != 0 ||
+	    sw_strcat(buf, SLASH, ARRAY_SIZE(buf)) != 0 ||
+	    sw_strcat(buf, "Downloads", ARRAY_SIZE(buf)) != 0 ||
+	    !is_directory(buf))
+		return (NULL);
+
+	return (sw_strdup(buf));
 #elif defined(WIN32)
-	char			buf[PATH_MAX] = { '\0' };
 	size_t			ret = 0;
 	static const char	var[] = "USERPROFILE";
 
