@@ -1,5 +1,5 @@
 /* Platform independent networking routines
-   Copyright (C) 2014-2024 Markus Uhlin. All rights reserved.
+   Copyright (C) 2014-2025 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -79,16 +79,13 @@ public:
 	long int	retries;
 	long int	retry;
 
-	reconnect_context()
-	{
-		this->backoff_delay	= 0;
-		this->delay		= 0;
-		this->delay_max		= 0;
-		this->retries		= 0;
-		this->retry		= 0;
-
-		this->initialized = false;
-	}
+	reconnect_context() : backoff_delay(0)
+	    , delay(0)
+	    , delay_max(0)
+	    , retries(0)
+	    , retry(0)
+	    , initialized(false)
+	{}
 
 	void
 	init(void)
@@ -176,7 +173,7 @@ check_conn_fail()
 }
 
 static void
-check_hostname(const char *host, PPRINTTEXT_CONTEXT ctx)
+check_hostname(CSTRING host, PPRINTTEXT_CONTEXT ctx)
 {
 	if (ssl_is_enabled() && config_bool("hostname_checking", true)) {
 		if (net_ssl_check_hostname(host, 0) != OK)
@@ -195,7 +192,7 @@ conn_check()
 		if (net_send("%cn", msglen) == -1)
 			return -1;
 	} else {
-		if (g_server_hostname != NULL) {
+		if (g_server_hostname != nullptr) {
 			if (net_send("PING %s", g_server_hostname) == -1)
 				return -1;
 		}
@@ -205,7 +202,7 @@ conn_check()
 }
 
 static inline void
-destroy_null_bytes(char *recvbuf, const int bytes_received)
+destroy_null_bytes(STRING recvbuf, const int bytes_received)
 {
 	int	i, j;
 
@@ -230,6 +227,33 @@ connect_hook(void)
 static void
 establish_conn(struct addrinfo *res, PPRINTTEXT_CONTEXT ctx)
 {
+	struct addrinfo *bind_res = nullptr;
+
+	if (res == nullptr)
+		return;
+	if (g_bind_hostname) {
+		int gai_ret;
+		struct addrinfo hints;
+
+		BZERO(&hints, sizeof hints);
+		hints.ai_flags     = res->ai_flags;
+		hints.ai_family    = res->ai_family;
+		hints.ai_socktype  = res->ai_socktype;
+		hints.ai_protocol  = 0;
+		hints.ai_addrlen   = 0;
+		hints.ai_addr      = nullptr;
+		hints.ai_canonname = nullptr;
+		hints.ai_next      = nullptr;
+
+		if ((gai_ret = getaddrinfo(g_cmdline_opts->hostname, nullptr,
+		    &hints, &bind_res)) != 0) {
+			printtext_print("err", "%s: getaddrinfo: %s: %s",
+			    __func__,
+			    g_cmdline_opts->hostname,
+			    gai_strerror(gai_ret));
+			return;
+		}
+	}
 	for (struct addrinfo *rp = res; rp; rp = rp->ai_next) {
 		if ((g_socket = socket(rp->ai_family, rp->ai_socktype,
 		    rp->ai_protocol)) == INVALID_SOCKET)
@@ -238,6 +262,20 @@ establish_conn(struct addrinfo *res, PPRINTTEXT_CONTEXT ctx)
 		net_set_recv_timeout(TEMP_RECV_TIMEOUT);
 		net_set_send_timeout(TEMP_SEND_TIMEOUT);
 
+		if (bind_res) {
+			errno = 0;
+
+			if (bind(g_socket, bind_res->ai_addr,
+			    bind_res->ai_addrlen) != 0) {
+				char strerrbuf[MAXERROR] = { '\0' };
+
+				printtext_print("err", "bind() error: %s",
+				    xstrerror(errno, strerrbuf, MAXERROR));
+				CLOSE_GLOBAL_SOCKET();
+				g_socket = INVALID_SOCKET;
+				break;
+			}
+		}
 		if (connect(g_socket, rp->ai_addr, rp->ai_addrlen) == 0) {
 			printtext(ctx, "%s", _("Connected!"));
 
@@ -251,24 +289,29 @@ establish_conn(struct addrinfo *res, PPRINTTEXT_CONTEXT ctx)
 			g_socket = INVALID_SOCKET;
 		}
 	} /* for */
+	if (bind_res)
+		freeaddrinfo(bind_res);
 }
 
 static int
 get_and_handle_remaining_bytes(const int bytes_remaining,
-    struct network_recv_context *ctx, const char *recvbuf, const int length)
+    struct network_recv_context *ctx, CSTRING recvbuf, const int length)
 {
-	char	*tmp = NULL;
-	char	*concat = NULL;
+	STRING	tmp    = nullptr;
+	STRING	concat = nullptr;
 
 	try {
 		int	bytes_received;
 		size_t	concatSize;
 
-		if (bytes_remaining <= 0 || ctx == NULL || recvbuf == NULL ||
-		    length < 0 || length > UCHAR_MAX)
+		if (bytes_remaining <= 0 ||
+		    ctx == nullptr ||
+		    recvbuf == nullptr ||
+		    length < 0 ||
+		    length > UCHAR_MAX)
 			throw std::runtime_error("invalid arguments");
 
-		tmp = static_cast<char *>(xmalloc(bytes_remaining + 1));
+		tmp = static_cast<STRING>(xmalloc(bytes_remaining + 1));
 		tmp[bytes_remaining] = '\0';
 
 		if ((bytes_received = net_recv(ctx, tmp, bytes_remaining)) !=
@@ -276,11 +319,11 @@ get_and_handle_remaining_bytes(const int bytes_remaining,
 			throw std::runtime_error("read bytes mismatch "
 			    "remaining");
 		}
-		if (memchr(tmp, 0, bytes_received) != NULL)
+		if (memchr(tmp, 0, bytes_received) != nullptr)
 			destroy_null_bytes(tmp, bytes_received);
 
 		concatSize = strlen(recvbuf) + strlen(tmp) + 1;
-		concat = static_cast<char *>(xmalloc(concatSize));
+		concat = static_cast<STRING>(xmalloc(concatSize));
 
 		if (sw_strcpy(concat, recvbuf, concatSize) == 0 &&
 		    sw_strcat(concat, tmp, concatSize) == 0)
@@ -301,10 +344,10 @@ get_and_handle_remaining_bytes(const int bytes_remaining,
 }
 
 static void
-get_ip_addresses(struct addrinfo *&res, const char *server, const char *port,
+get_ip_addresses(struct addrinfo *&res, CSTRING server, CSTRING port,
     PPRINTTEXT_CONTEXT ctx)
 {
-	if ((res = net_addr_resolve(server, port)) == NULL) {
+	if ((res = net_addr_resolve(server, port)) == nullptr) {
 		throw std::runtime_error(_("Unable to get a list of IP "
 		    "addresses"));
 	} else {
@@ -313,7 +356,7 @@ get_ip_addresses(struct addrinfo *&res, const char *server, const char *port,
 }
 
 static void
-handle_conn_err(PPRINTTEXT_CONTEXT ptext_ctx, const char *what,
+handle_conn_err(PPRINTTEXT_CONTEXT ptext_ctx, CSTRING what,
     long int *sleep_time_seconds, conn_res_t &conn_res)
 {
 	ptext_ctx->spec_type = TYPE_SPEC1_FAILURE;
@@ -349,7 +392,7 @@ handle_conn_err(PPRINTTEXT_CONTEXT ptext_ctx, const char *what,
 }
 
 static void
-save_last_server(const char *server, const char *port, const char *password)
+save_last_server(CSTRING server, CSTRING port, CSTRING password)
 {
 	int ret;
 
@@ -520,10 +563,10 @@ net_connect(const struct network_connect_context *ctx,
 {
 	PRINTTEXT_CONTEXT	 ptext_ctx;
 	conn_res_t		 conn_res = CONNECTION_FAILED;
-	struct addrinfo		*res = NULL;
+	struct addrinfo		*res = nullptr;
 	struct integer_context	 intctx("identd_port", 113, 65535, 113);
 
-	if (ctx == NULL || sleep_time_seconds == NULL)
+	if (ctx == nullptr || sleep_time_seconds == nullptr)
 		err_exit(EINVAL, "%s", __func__);
 	else if (atomic_load_bool(&g_connection_in_progress))
 		return CONNECTION_FAILED;
@@ -606,7 +649,7 @@ net_connect(const struct network_connect_context *ctx,
 }
 
 int
-net_send_fake(const char *fmt, ...)
+net_send_fake(CSTRING fmt, ...)
 {
 	int bytes_sent;
 	va_list ap;
@@ -619,13 +662,13 @@ net_send_fake(const char *fmt, ...)
 }
 
 struct addrinfo *
-net_addr_resolve(const char *host, const char *port)
+net_addr_resolve(CSTRING host, CSTRING port)
 {
-	struct addrinfo *res = NULL;
+	struct addrinfo *res = nullptr;
 	struct addrinfo hints;
 
-	if (host == NULL || port == NULL)
-		return NULL;
+	if (host == nullptr || port == nullptr)
+		return nullptr;
 
 	BZERO(&hints, sizeof hints);
 	hints.ai_flags     = AI_CANONNAME;
@@ -633,22 +676,22 @@ net_addr_resolve(const char *host, const char *port)
 	hints.ai_socktype  = SOCK_STREAM;
 	hints.ai_protocol  = 0;
 	hints.ai_addrlen   = 0;
-	hints.ai_addr      = NULL;
-	hints.ai_canonname = NULL;
-	hints.ai_next      = NULL;
+	hints.ai_addr      = nullptr;
+	hints.ai_canonname = nullptr;
+	hints.ai_next      = nullptr;
 
-	return (getaddrinfo(host, port, &hints, &res) != 0 ? NULL : res);
+	return (getaddrinfo(host, port, &hints, &res) != 0 ? nullptr : res);
 }
 
 struct server *
-server_new(const char *host, const char *port, const char *pass)
+server_new(CSTRING host, CSTRING port, CSTRING pass)
 {
 	struct server *server =
 	    static_cast<struct server *>(xmalloc(sizeof *server));
 
 	server->host = sw_strdup(host);
 	server->port = sw_strdup(port);
-	server->pass = (pass ? sw_strdup(pass) : NULL);
+	server->pass = (pass ? sw_strdup(pass) : nullptr);
 
 	return server;
 }
@@ -661,13 +704,13 @@ net_connect_clean_up(void)
 }
 
 void
-destroy_null_bytes_exported(char *buf, const int len)
+destroy_null_bytes_exported(STRING buf, const int len)
 {
 	destroy_null_bytes(buf, len);
 }
 
 static int
-icb(int &bytes_received, struct network_recv_context *ctx, char *recvbuf)
+icb(int &bytes_received, struct network_recv_context *ctx, STRING recvbuf)
 {
 	char	array[10] = { '\0' };
 	int	length, ret;
@@ -698,7 +741,7 @@ icb(int &bytes_received, struct network_recv_context *ctx, char *recvbuf)
 			return ERR;
 		}
 	} else if (bytes_received > 0) {
-		if (memchr(recvbuf, 0, bytes_received) != NULL)
+		if (memchr(recvbuf, 0, bytes_received) != nullptr)
 			destroy_null_bytes(recvbuf, bytes_received);
 
 		try {
@@ -712,13 +755,13 @@ icb(int &bytes_received, struct network_recv_context *ctx, char *recvbuf)
 }
 
 static void
-irc(int &bytes_received, struct network_recv_context *ctx, char *recvbuf,
+irc(int &bytes_received, struct network_recv_context *ctx, STRING recvbuf,
     char **message_concat, enum message_concat_state *state)
 {
 	if ((bytes_received = net_recv(ctx, recvbuf, RECVBUF_SIZE)) == -1) {
 		(void) atomic_swap_bool(&g_connection_lost, true);
 	} else if (bytes_received > 0) {
-		if (memchr(recvbuf, 0, bytes_received) != NULL)
+		if (memchr(recvbuf, 0, bytes_received) != nullptr)
 			destroy_null_bytes(recvbuf, bytes_received);
 
 		try {
@@ -734,8 +777,8 @@ void
 net_irc_listen(bool *connection_lost)
 {
 	PRINTTEXT_CONTEXT		 ptext_ctx;
-	char				*message_concat = NULL;
-	char				*recvbuf = NULL;
+	STRING				 recvbuf = nullptr;
+	char				*message_concat = nullptr;
 	enum message_concat_state	 state = CONCAT_BUFFER_IS_EMPTY;
 	int				 bytes_received = -1;
 	struct network_recv_context	 ctx(g_socket, 0, 5, 0);
@@ -748,7 +791,7 @@ net_irc_listen(bool *connection_lost)
 	block_signals();
 	*connection_lost = false;
 	atomic_swap_bool(&g_connection_lost, false);
-	recvbuf = static_cast<char *>(xmalloc(RECVBUF_SIZE + 1));
+	recvbuf = static_cast<STRING>(xmalloc(RECVBUF_SIZE + 1));
 	recvbuf[RECVBUF_SIZE] = '\0';
 	irc_init();
 	netsplit_init();
@@ -849,7 +892,7 @@ net_request_disconnect(void)
 void
 server_destroy(struct server *server)
 {
-	if (server == NULL)
+	if (server == nullptr)
 		return;
 	free(server->host);
 	free(server->port);
