@@ -12,7 +12,7 @@ void	statusbar_hide(void);
 
 void	statusbar_recreate(int rows, int cols);
 void	statusbar_top_panel(void);
-void	statusbar_update_display_beta(void);
+void	statusbar_update(void);
 __SWIRC_END_DECLS
 
 #endif
