@@ -27,6 +27,9 @@ errno_t		textBuf_ins_prev(PTEXTBUF, PTEXTBUF_ELMT, const char *text,
 		    int indent);
 errno_t		textBuf_remove(PTEXTBUF, PTEXTBUF_ELMT);
 void		textBuf_destroy(PTEXTBUF);
+void		textBuf_emplace_back(const char *fn, PTEXTBUF, const char *text,
+		    int indent);
+void		textBuf_pop_head(const char *fn, PTEXTBUF);
 __SWIRC_END_DECLS
 
 /* Inline function definitions
@@ -47,7 +50,7 @@ textBuf_head(const TEXTBUF *buf)
 static SW_INLINE PTEXTBUF_ELMT
 textBuf_tail(const TEXTBUF *buf)
 {
-	return (buf->tail);
+	return (buf->tail); // NOLINT: false positive
 }
 
 #endif
