@@ -94,6 +94,14 @@ const int	g_scroll_amount = 6;
 _Atomic(int)	g_ntotal_windows = 0;
 volatile bool	g_redrawing_window = false;
 
+#if defined(UNIX)
+pthread_mutex_t		g_actwin_mtx;
+pthread_mutex_t		g_win_htbl_mtx;
+#elif defined(WIN32)
+HANDLE			g_actwin_mtx;   // Active window mutex
+HANDLE			g_win_htbl_mtx; // Window hash table mutex
+#endif
+
 /* Objects with internal linkage
    ============================= */
 
@@ -118,7 +126,7 @@ change_window(PIRC_WINDOW window)
 
 	if (window == NULL)
 		return ENOENT; /* window not found */
-	else if (window == g_active_window)
+	else if (is_the_active_window(window))
 		return 0; /* window already active */
 	else if (top_panel(window->pan) == ERR)
 		return EPERM;
@@ -126,9 +134,11 @@ change_window(PIRC_WINDOW window)
 	if (window->nicklist.pan != NULL)
 		(void) top_panel(window->nicklist.pan);
 
+	mutex_lock(&g_actwin_mtx);
 	g_active_window = window;
+	mutex_unlock(&g_actwin_mtx);
 	titlebar(" %s ", (window->title != NULL ? window->title : ""));
-	statusbar_update_display_beta();
+	statusbar_update();
 
 	if ((pwin = readline_get_active_pwin()) != NULL) {
 		STRING prompt;
@@ -312,6 +322,8 @@ reassign_window_refnums(void)
 {
 	int ref_count = 1;
 
+	mutex_lock(&g_win_htbl_mtx);
+
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			/*
@@ -323,6 +335,8 @@ reassign_window_refnums(void)
 		}
 	}
 
+	mutex_unlock(&g_win_htbl_mtx);
+
 	sw_assert(g_status_window->refnum == 1);
 	sw_assert(ref_count == g_ntotal_windows);
 }
@@ -382,7 +396,7 @@ window_redraw(PIRC_WINDOW window, const int rows, const int pos,
 		(void) atomic_swap_bool(&g_redrawing_window, false);
 	}
 
-	statusbar_update_display_beta();
+	statusbar_update();
 	readline_top_panel();
 }
 
@@ -469,17 +483,40 @@ window_recreate(PIRC_WINDOW window, int rows, int cols)
 	    false);
 }
 
+static void
+create_mutexes(void)
+{
+	mutex_new(&g_actwin_mtx);
+	mutex_new(&g_win_htbl_mtx);
+}
+
 void
 windowSystem_init(void)
 {
+#if defined(UNIX)
+	static pthread_once_t	init_done = PTHREAD_ONCE_INIT;
+#elif defined(WIN32)
+	static init_once_t	init_done = ONCE_INITIALIZER;
+#endif
+
 #if defined(UNIX) && USE_LIBNOTIFY
 	if (!notify_init("Swirc IRC client"))
 		err_log(0, "%s: notify_init: error", __func__);
 #endif
 
+#if defined(UNIX)
+	if ((errno = pthread_once(&init_done, create_mutexes)) != 0)
+		err_sys("%s: pthread_once", __func__);
+#elif defined(WIN32)
+	if ((errno = init_once(&init_done, create_mutexes)) != 0)
+		err_sys("%s: init_once", __func__);
+#endif
+
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		*entry_p = NULL;
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 
 	g_status_window = g_active_window = NULL;
 	g_ntotal_windows = 0;
@@ -498,16 +535,21 @@ windowSystem_deinit(void)
 {
 	PIRC_WINDOW p, tmp;
 
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		for (p = *entry_p; p != NULL; p = tmp) {
 			tmp = p->next;
 			hUndef(p);
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 
 #if defined(UNIX) && USE_LIBNOTIFY
 	notify_uninit();
 #endif
+
+	mutex_destroy(&g_actwin_mtx);
+	mutex_destroy(&g_win_htbl_mtx);
 }
 
 static void
@@ -530,6 +572,7 @@ make_window_title(CSTRING nick)
 		return addrof(title[0]);
 	}
 
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			PNAMES n;
@@ -543,9 +586,11 @@ make_window_title(CSTRING nick)
 				continue;
 			(void) snprintf(title, sizeof title, "%s (%s): %s",
 			    nick, n->account, n->rl_name);
+			mutex_unlock(&g_win_htbl_mtx);
 			return addrof(title[0]);
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 
 	set_nick_as_title(title, nick, sizeof title);
 	return addrof(title[0]);
@@ -563,11 +608,15 @@ window_by_label(CSTRING label)
 	if (label == NULL || strings_match(label, ""))
 		return NULL;
 
+	mutex_lock(&g_win_htbl_mtx);
 	for (window = hash_table[hash(label)]; window != NULL;
 	     window = window->next) {
-		if (strings_match_ignore_case(label, window->label))
+		if (strings_match_ignore_case(label, window->label)) {
+			mutex_unlock(&g_win_htbl_mtx);
 			return window;
+		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 
 	return NULL;
 }
@@ -581,12 +630,18 @@ window_by_refnum(int refnum)
 {
 	if (refnum < 1 || refnum > g_ntotal_windows)
 		return NULL;
+
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
-			if (refnum == window->refnum)
+			if (refnum == window->refnum) {
+				mutex_unlock(&g_win_htbl_mtx);
 				return window;
+			}
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
+
 	return NULL;
 }
 
@@ -596,6 +651,7 @@ get_list_of_matching_channels(CSTRING search_var)
 	PTEXTBUF	matches = textBuf_new();
 	const size_t	varlen = strlen(search_var);
 
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			if (is_irc_channel(window->label) &&
@@ -605,10 +661,13 @@ get_list_of_matching_channels(CSTRING search_var)
 			}
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
+
 	if (textBuf_size(matches) == 0) {
 		textBuf_destroy(matches);
 		return NULL;
 	}
+
 	return matches;
 }
 
@@ -618,6 +677,7 @@ get_list_of_matching_queries(CSTRING search_var)
 	PTEXTBUF	matches = textBuf_new();
 	const size_t	varlen = strlen(search_var);
 
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			if (!is_irc_channel(window->label) &&
@@ -628,13 +688,28 @@ get_list_of_matching_queries(CSTRING search_var)
 			}
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
+
 	if (textBuf_size(matches) == 0) {
 		textBuf_destroy(matches);
 		return NULL;
 	}
+
 	return matches;
 }
 
+bool
+is_the_active_window(PIRC_WINDOW which)
+{
+	bool yesno;
+
+	mutex_lock(&g_actwin_mtx);
+	yesno = (which == g_active_window);
+	mutex_unlock(&g_actwin_mtx);
+
+	return yesno;
+}
+
 /**
  * Change window to the one specified by @label
  */
@@ -667,7 +742,9 @@ destroy_chat_window(CSTRING label)
 	else if ((window = window_by_label(label)) == NULL)
 		return ENOENT;
 
+	mutex_lock(&g_win_htbl_mtx);
 	hUndef(window);
+	mutex_unlock(&g_win_htbl_mtx);
 	reassign_window_refnums();
 	const errno_t ret = change_window_by_refnum(g_ntotal_windows);
 	(void) ret;
@@ -705,7 +782,9 @@ spawn_chat_window(CSTRING label, CSTRING title)
 	inst_ctx.pan    = term_new_panel(LINES - 2, 0, 1, 0);
 	inst_ctx.refnum = g_ntotal_windows + 1;
 
+	mutex_lock(&g_win_htbl_mtx);
 	entry = hInstall(&inst_ctx);
+	mutex_unlock(&g_win_htbl_mtx);
 	apply_window_options(panel_window(entry->pan));
 	const errno_t ret = change_window_by_label(entry->label);
 	(void) ret;
@@ -748,6 +827,8 @@ window_close_all_priv_conv(void)
 	char	*priv_conv[200] = { NULL };
 	size_t	 pc_assigned = 0;
 
+	mutex_lock(&g_win_htbl_mtx);
+
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			if (window == g_status_window ||
@@ -764,6 +845,7 @@ window_close_all_priv_conv(void)
 	}
 
   out_of_both_loops:
+	mutex_unlock(&g_win_htbl_mtx);
 
 	if (pc_assigned == 0) {
 		(void) napms(222);
@@ -783,6 +865,7 @@ window_close_all_priv_conv(void)
 void
 window_foreach_destroy_names(void)
 {
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			if (is_irc_channel(window->label)) {
@@ -798,6 +881,7 @@ window_foreach_destroy_names(void)
 			}
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 }
 
 /**
@@ -806,12 +890,14 @@ window_foreach_destroy_names(void)
 void
 window_foreach_rejoin_all_channels(void)
 {
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			if (is_irc_channel(window->label))
 				(void) net_send("JOIN %s", window->label);
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 }
 
 void
@@ -980,9 +1066,11 @@ windows_list_all(void)
 void
 windows_recreate_all(int rows, int cols)
 {
+	mutex_lock(&g_win_htbl_mtx);
 	FOREACH_HASH_TABLE_ENTRY() {
 		FOREACH_WINDOW_IN_ENTRY() {
 			window_recreate(window, rows, cols);
 		}
 	}
+	mutex_unlock(&g_win_htbl_mtx);
 }
