@@ -12,6 +12,7 @@
 #endif
 
 #include "atomicops.h"
+#include "mutex.h"
 #include "textBuffer.h"
 
 #define ACTWINLABEL g_active_window->label
@@ -76,6 +77,14 @@ extern const int	g_scroll_amount;
 extern _Atomic(int)	g_ntotal_windows;
 extern volatile bool	g_redrawing_window;
 
+#if defined(UNIX)
+extern pthread_mutex_t	g_actwin_mtx;
+extern pthread_mutex_t	g_win_htbl_mtx;
+#elif defined(WIN32)
+extern HANDLE		g_actwin_mtx;
+extern HANDLE		g_win_htbl_mtx;
+#endif
+
 void windowSystem_init(void);
 void windowSystem_deinit(void);
 
@@ -89,6 +98,7 @@ PIRC_WINDOW	window_by_label(CSTRING);
 PIRC_WINDOW	window_by_refnum(int);
 PTEXTBUF	get_list_of_matching_channels(CSTRING search_var);
 PTEXTBUF	get_list_of_matching_queries(CSTRING search_var);
+bool		is_the_active_window(PIRC_WINDOW);
 errno_t		change_window_by_label(CSTRING);
 errno_t		change_window_by_refnum(int);
 errno_t		destroy_chat_window(CSTRING label);
